@@ -255,4 +255,71 @@ impl super::Kubectl {
         let lp = self.list_params();
         self.crds().list(&lp).await.map(|object| object.items)
     }
+
+    pub(super) async fn ensure_global_k_is_installed_impl<K>(
+        &self,
+        mut object: K,
+        force: bool,
+    ) -> kube::Result<K>
+    where
+        K: kube::ResourceExt + ser::Serialize + de::DeserializeOwned + Clone + fmt::Debug,
+        <K as kube::Resource>::DynamicType: Default,
+    {
+        let name = object.name();
+        let objects = self.api::<K>();
+        if let Ok(existing) = objects.get(&name).await {
+            object.meta_mut().resource_version = existing.resource_version();
+        }
+
+        if force {
+            self.force_patch(object, objects).await
+        } else {
+            self.patch(object, objects).await
+        }
+    }
+
+    pub(crate) async fn ensure_namespaced_k_is_installed_impl<K>(
+        &self,
+        mut object: K,
+        namespace: &str,
+        force: bool,
+    ) -> kube::Result<K>
+    where
+        K: kube::Resource + ser::Serialize + de::DeserializeOwned + Clone + fmt::Debug,
+        <K as kube::Resource>::DynamicType: Default,
+    {
+        let name = object.name();
+        let objects = self.namespaced_api::<K>(namespace);
+        if let Ok(existing) = objects.get(&name).await {
+            object.meta_mut().resource_version = existing.resource_version();
+        }
+
+        if force {
+            self.force_patch(object, objects).await
+        } else {
+            self.patch(object, objects).await
+        }
+    }
+
+    pub(crate) async fn ensure_default_namespaced_k_is_installed_impl<K>(
+        &self,
+        mut object: K,
+        force: bool,
+    ) -> kube::Result<K>
+    where
+        K: kube::Resource + ser::Serialize + de::DeserializeOwned + Clone + fmt::Debug,
+        <K as kube::Resource>::DynamicType: Default,
+    {
+        let name = object.name();
+        let objects = self.default_namespaced_api::<K>();
+        if let Ok(existing) = objects.get(&name).await {
+            object.meta_mut().resource_version = existing.resource_version();
+        }
+
+        if force {
+            self.force_patch(object, objects).await
+        } else {
+            self.patch(object, objects).await
+        }
+    }
 }
